clc; clear; close all

n = 64;
m = n/8; % number of direction between 0 and pi/4
thetas = pi/4*(1/2/m:1/m:8-1/2/m);

xs = -1+1/n:2/n:1-1/n;

%--- characteristic function of the domain
domain = myMap('disk',n,1,0,0);

%--- intrisic attenuation
% att = zeros(n);
att = myMap('mickey',n,0.02,12,0.2);%+0.3*domain;
% att = myMap('nicebumps',n,0.1,5,0.5);%+0.3*domain;
% att = domain;

%--- scattering
coeffScat = 0.2;
% scat = coeffScat*myMap('gaussian',n,1.5/n,0.5,0.5);
% scat = coeffScat*myMap('mickey',n,0.01,16,0.2);
scat = coeffScat*myMap('nicebumps',n,0.1,3,0.4);
% scat = coeffScat*myMap('disk',n,0.03,0.5,0)...
%     + coeffScat*myMap('disk',n,0.03,-0.5,0);
% scat = coeffScat*(myMap('disk',n,0.5,0,0)...
%     -myMap('disk',n,0.4,0,0));
% scat = coeffScat*myMap('disk',n,0.02,0,0);
% scat = coeffScat*myMap('phantom',n,0.06,3,0.5);
% scat = coeffScat*(myMap('gaussian',n,0.3,0,0) + ...
%     myMap('gaussian',n,0.125,0.5,0.5) + ...
%     myMap('gaussian',n,0.125,-0.5,-0.5));
% scat = zeros(n);

% cropping the coefficients to the domain
scat = scat.*domain;

nIter = 0; % number of iterations for the reconstruction scheme
nScat = 1; % number of scattering terms for the forward solver

%--- indices thIn, thOut
% indsIn = [1,1,1,1];
% indsOut = indsIn + [m/4,ceil(2*m/3),ceil(4*m/3),2*m];
indsIn = 1;
indsOut = indsIn + m;

%--- input options
opts.input.type = 'dirac';
opts.input.sig = 4.5/n;

%--- diffusion types
%     opts.diff.type = 'linear'; opts.diff.d = tan(pi/4/m)^2;
   % opts.diff.type = 'FPB'; opts.diff.d = 0.00;
   %  opts.diff.type = 'none'; opts.diff.sig = 0;

%--- domain for diffusion
opts.diff.domain = domain;
%--- recontruction formula for attenation
opts.diff.rc = 'intCorr'; % 'int', 'peak', 'intCorr', 'peakCorr'
    
% corrector for scattering reconstruction based 
% on unitary scattering coefficient
opts.corr = 0;

%--- options for the inverse Radon Transform    
opts.IRT.filter = 'ram-lak'; % filter for the inverse Radon Transform
opts.IRT.band = 0.9; % frequency truncation for the IRT

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% MAIN


% %% ============= attenuation reconstruction from ballistic and all 
% %%============= reconstruction formulae

    %--- parameters for the inverse Radon Transform
    filter = opts.IRT.filter;
    band = opts.IRT.band;
    %--- number of radii for RT, make it even
    nrRT = ceil(n*sqrt(2)) + mod(ceil(n*sqrt(2)),2);
    %--- offset for the RT data
    os = (nrRT-n)/2;

    %--- diffusion types
%     opts.diff.type = 'linear'; opts.diff.d = tan(pi/4/m)^2;
    opts.diff.type = 'FPB'; opts.diff.d = 0.0010;
%     opts.diff.type = 'none'; opts.diff.sig = 0;
    
    angles = 180/pi*(thetas-pi/2);
    %--- options for diffusion and reconstruction
    diff = makeDiff(n,opts.diff);
  
    %--- corrector
    disp('ballistic measurements without attenuation')
    corr = measForRcAtt(n,m,thetas,zeros(n),zeros(n),0,diff,0,opts.input);
    
    %--- measurements
    D = measForRcAtt(n,m,thetas,att,zeros(n),0,diff,0,opts.input);
    
    %--- reconstructions
    opts.diff.rc = 'int';
    RT1 = [zeros(os,8*m); measToRT(n,m,D,opts); zeros(os,8*m)];
    opts.diff.rc = 'peak';
    RT2 = [zeros(os,8*m); measToRT(n,m,D,opts); zeros(os,8*m)];
    
    rc1 = (n/2)*myIRadon(RT1,filter,angles,band,n);
    rc2 = (n/2)*myIRadon(RT2,filter,angles,band,n);    
    
    opts.diff.rc = 'intCorr';
    corrInt = measToRT(n,m,corr,opts);
    RT3 = [zeros(os,8*m); measToRT(n,m,D,opts)-corrInt; zeros(os,8*m)];
    opts.diff.rc = 'peakCorr';
    corrPeak = measToRT(n,m,corr,opts);
    RT4 = [zeros(os,8*m); measToRT(n,m,D,opts)-corrPeak; zeros(os,8*m)];
    
    rc3 = (n/2)*myIRadon(RT3,filter,angles,band,n);
    rc4 = (n/2)*myIRadon(RT4,filter,angles,band,n);
    
    opts.diff.rc = 'nodiff';
    RT5 = [zeros(os,8*m); measToRT(n,m,D,opts); zeros(os,8*m)];
    
    rc5 = (n/2)*myIRadon(RT5,filter,angles,band,n);    
    
    
    %--- display
%     figure;
%     subplot(1,5,1); dispmat(RT1,'integral')
%     subplot(1,5,2); dispmat(RT2,'peak')
%     subplot(1,5,3); dispmat(RT3,'integral corrected')
%     subplot(1,5,4); dispmat(RT4,'peak corrected')
%     subplot(1,5,5); dispmat(RT5,'no diff')

%     figure; plot([RT1(:,1),RT2(:,1),RT3(:,1),RT4(:,1),RT5(:,1)])
%     legend('int','peak','int corr','peak corr','no diff')
    
    err1 = num2str(100*norm(rc1-att,'fro')/norm(att,'fro'));
    err2 = num2str(100*norm(rc2-att,'fro')/norm(att,'fro'));
    err3 = num2str(100*norm(rc3-att,'fro')/norm(att,'fro'));
    err4 = num2str(100*norm(rc4-att,'fro')/norm(att,'fro'));
    err5 = num2str(100*norm(rc5-att,'fro')/norm(att,'fro'));
    
    figure;
    subplot(2,3,1); dispmat(rc1,['integral, err = ',err1,' %'])
    subplot(2,3,2); dispmat(rc2,['peak, err = ',err2,' %'])
    subplot(2,3,3); dispmat(rc3,['integral corrected, err = ',err3,' %'])
    subplot(2,3,4); dispmat(rc4,['peak corrected, err = ',err4,' %'])
    subplot(2,3,5); dispmat(rc5,['no diff, err = ',err5,' %'])
    
    subplot(2,3,6); 
    plot(xs,...
        [rc1(:,n/2),rc2(:,n/2),rc3(:,n/2),rc4(:,n/2),rc5(:,n/2),...
        att(:,n/2)],'LineWidth',1.5);
    legend('int','peak','int corr','peak corr','no diff','exact')
