clc; clear; close all

n = 32;
m = n/8; % number of direction between 0 and pi/4
thetas = pi/4*(1/2/m:1/m:8-1/2/m);

xs = -1+1/n:2/n:1-1/n;

%--- characteristic function of the domain
domain = myMap('disk',n,1,0,0);

%--- intrisic attenuation
% att = zeros(n);
att = myMap('mickey',n,0.02,12,0.2);%+0.3*domain;
% att = myMap('nicebumps',n,0.1,5,0.5);%+0.3*domain;
% att = domain;

%--- scattering
coeffScat = 0.2;
% scat = coeffScat*myMap('gaussian',n,1.5/n,0.5,0.5);
% scat = coeffScat*myMap('mickey',n,0.01,16,0.2);
scat = coeffScat*myMap('nicebumps',n,0.1,3,0.4);
% scat = coeffScat*myMap('disk',n,0.03,0.5,0)...
%     + coeffScat*myMap('disk',n,0.03,-0.5,0);
% scat = coeffScat*(myMap('disk',n,0.5,0,0)...
%     -myMap('disk',n,0.4,0,0));
% scat = coeffScat*myMap('disk',n,0.02,0,0);
% scat = coeffScat*myMap('phantom',n,0.06,3,0.5);
% scat = coeffScat*(myMap('gaussian',n,0.3,0,0) + ...
%     myMap('gaussian',n,0.125,0.5,0.5) + ...
%     myMap('gaussian',n,0.125,-0.5,-0.5));
% scat = zeros(n);

% cropping the coefficients to the domain
scat = scat.*domain;

nIter = 2; % number of iterations for the reconstruction scheme
nScat = 1; % number of scattering terms for the forward solver

%--- indices thIn, thOut
% indsIn = [1,1,1,1];
% indsOut = indsIn + [m/4,ceil(2*m/3),ceil(4*m/3),2*m];
indsIn = 1;
indsOut = indsIn + m;

%--- input options
opts.input.type = 'dirac';
opts.input.sig = 4.5/n;

%--- diffusion types
%     opts.diff.type = 'linear'; opts.diff.d = tan(pi/4/m)^2;
  %  opts.diff.type = 'FPB'; opts.diff.d = 0.00;
     opts.diff.type = 'none'; opts.diff.sig = 0;

%--- domain for diffusion
opts.diff.domain = domain;
%--- recontruction formula for attenation
opts.diff.rc = 'intCorr'; % 'int', 'peak', 'intCorr', 'peakCorr'
    
% corrector for scattering reconstruction based 
% on unitary scattering coefficient
opts.corr = 0;

%--- options for the inverse Radon Transform    
opts.IRT.filter = 'ram-lak'; % filter for the inverse Radon Transform
opts.IRT.band = 0.9; % frequency truncation for the IRT

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% MAIN
% 
% figure; 
% subplot(1,3,1); dispmat(scat,'scattering');
% subplot(1,3,2); dispmat(att,'intrisic attenuation');
% subplot(1,3,3); dispmat(att+2*pi*scat,'total attenuation');
% 
% %% ============ one series of measurements
%     attRot = rotateImages(att+2*pi*scat,thetas(1:2*m));
% 
%     diff = makeDiff(n,opts.diff);
%     
%     % localisation of the input ray
%     indR = ceil(2*n/3); indTheta = m/2;
%     
%     input = makeInput(n,m,indR,indTheta,opts.input);
%     
%     tic
%     meas = measurements(n,m,thetas,attRot,scat,diff,input,nScat,0);
%     toc
%     % --- display
%     rs = -1+1/n:2/n:1-1/n;
%     figure;
%     subplot(2,3,1); dispmat(att,'attenuation')
%     subplot(2,3,2); dispmat(scat,'scattering')
%     subplot(2,3,3); plot(rs,meas(:,indTheta,1)); 
%     title('ballistic part at \theta=\pi/8'); xlabel('r')
%     subplot(2,3,4); dispmat(meas(:,:,2),'single scattering'); 
%     xlabel('\theta'); ylabel('r')
%     subplot(2,3,5); dispmat(meas(:,:,3),'double scattering'); 
%     xlabel('\theta'); ylabel('r')
%     subplot(2,3,6); dispmat(meas(:,:,4),'triple scattering'); 
%     xlabel('\theta'); ylabel('r')

% % =========== iterated scattering reconstruction
% 
%     %--- compute initial guess : yes
%     opts.computeIG = 1;
%     %--- type of iterative scheme ('initial' or 'modified')
%     opts.typeIter = 'modified';
%         
%     %--- reconstructions without calibration
%     [scatRc, scatErr, attRc, attErr] = itReconstrScat(n,m,thetas,...
%         att,scat,nScat,nIter,indsIn,indsOut,opts);
% 
%     %--- reconstructions with calibration
%     opts.corr = 1;
% 
%     [scatRc2, relErr2, attRc2, attErr2] = itReconstrScat(n,m,thetas,...
%         att,scat,nScat,nIter,indsIn,indsOut,opts);
%     
%     %--- display
%     figure
%     subplot(1,4,1); dispmat(scatRc(:,:,end), 'A: without corrector')
%     subplot(1,4,2); dispmat(scatRc2(:,:,end), 'B: with corrector')
%     subplot(1,4,3); plot(xs,[scatRc(:,n/2,end),scatRc2(:,n/2,end),scat(:,n/2)],...
%         'LineWidth',1.5); title('C: cut at x=0')
%     legend('no corrector','corrector','exact')
%     subplot(1,4,4);
%     plot(xs,[scatRc(n/2,:,end).',scatRc2(n/2,:,end).',scat(n/2,:).'],...
%         'LineWidth',1.5); title('D: cut at y=0')
%     legend('no corrector','corrector','exact')

% % %% ============ iterated attenuation reconstruction
    [att,attRc,RTs,attErr] = itReconstrAtt(n,m,thetas,att,scat,nScat,...
        nIter,opts);

    relErr = norm(att-squeeze(attRc(:,:,end)),'fro')/norm(att,'fro');
    
    figure;
    subplot(2,3,1); dispmat(att,'attenuation');
    subplot(2,3,2); dispmat(squeeze(RTs(:,:,end)),'approximate Radon transform')
    subplot(2,3,3); dispmat(squeeze(attRc(:,:,end)),'reconstructed');
    subplot(2,3,4); dispmat((abs(att-squeeze(attRc(:,:,end))))); 
    subplot(2,3,5); plot(xs,att(n/2,:),'b',xs,attRc(n/2,:,end),'r')
    title('cut at y=0'); legend('exact','reconstr.')
    subplot(2,3,6); plot(xs,att(:,n/2),'b',xs,attRc(:,n/2,end),'r')
    title('cut at x=0'); legend('exact','reconstr.')
    
    figure;
    for it = 1:nIter,
        subplot(2,nIter+1,it); dispmat(attRc(:,:,it));
        subplot(2,nIter+1,nIter+1+it); dispmat(abs(att-attRc(:,:,it)));
    end
    subplot(2,nIter+1,nIter+1); dispmat(attRc(:,:,end));
    subplot(2,nIter+1,2*nIter+2); plot(attErr)

% %% ============ joint iterated attenuation/scattering reconstruction
%     [attRc,scatRc] = itReconstrAttScat(n,m,thetas,att,scat,nScat,...
%         nIter, indsIn,indsOut,opts);


% %% ============= attenuation reconstruction from ballistic and all 
% %%============= reconstruction formulae
% 
%     %--- parameters for the inverse Radon Transform
%     filter = opts.IRT.filter;
%     band = opts.IRT.band;
%     %--- number of radii for RT, make it even
%     nrRT = ceil(n*sqrt(2)) + mod(ceil(n*sqrt(2)),2);
%     %--- offset for the RT data
%     os = (nrRT-n)/2;
% 
%     angles = 180/pi*(thetas-pi/2);
%     %--- options for diffusion and reconstruction
%     diff = makeDiff(n,opts.diff);
% 
%     %--- corrector
%     disp('ballistic measurements without attenuation')
%     corr = measForRcAtt(n,m,thetas,zeros(n),zeros(n),0,diff,0,opts.input);
%     
%     %--- measurements
%     D = measForRcAtt(n,m,thetas,att,zeros(n),0,diff,0,opts.input);
%     
%     %--- reconstructions
%     opts.diff.rc = 'int';
%     RT1 = [zeros(os,8*m); ...
%         measToRT(n,m,D,opts.diff,opts.input); zeros(os,8*m)];
%     opts.diff.rc = 'peak';
%     RT2 = [zeros(os,8*m); ...
%         measToRT(n,m,D,opts.diff,opts.input); zeros(os,8*m)];
%     
%     rc1 = (n/2)*myIRadon(RT1,filter,angles,band,n);
%     rc2 = (n/2)*myIRadon(RT2,filter,angles,band,n);    
%     
%     
%     opts.diff.rc = 'intCorr';
%     corrInt = measToRT(n,m,corr,opts.diff,opts.input);
%     RT3 = [zeros(os,8*m); ... 
%         measToRT(n,m,D,opts.diff,opts.input)-corrInt; zeros(os,8*m)];
%     opts.diff.rc = 'peakCorr';
%     corrPeak = measToRT(n,m,corr,opts.diff,opts.input);
%     RT4 = [zeros(os,8*m); ...
%         measToRT(n,m,D,opts.diff,opts.input)-corrPeak; zeros(os,8*m)];
%     
%     rc3 = (n/2)*myIRadon(RT3,filter,angles,band,n);
%     rc4 = (n/2)*myIRadon(RT4,filter,angles,band,n);
%     
%     %--- display
%     figure;
%     subplot(2,4,1); dispmat(RT1,'integral')
%     subplot(2,4,2); dispmat(RT2,'peak')
%     subplot(2,4,3); dispmat(RT3,'integral corrected')
%     subplot(2,4,4); dispmat(RT4,'peak corrected')
%     subplot(2,4,5); plot(RT1(:,1));
%     subplot(2,4,6); plot(RT2(:,1));
%     subplot(2,4,7); plot(RT3(:,1));
%     subplot(2,4,8); plot(RT4(:,1));
%     
%     figure; plot([RT1(:,1),RT2(:,1),RT3(:,1),RT4(:,1)])
%     
%     err1 = num2str(100*norm(rc1-att,'fro')/norm(att,'fro'));
%     err2 = num2str(100*norm(rc2-att,'fro')/norm(att,'fro'));
%     err3 = num2str(100*norm(rc3-att,'fro')/norm(att,'fro'));
%     err4 = num2str(100*norm(rc4-att,'fro')/norm(att,'fro'));
%     
%     figure;
%     subplot(1,5,1); dispmat(rc1,['integral, err = ',err1,' %'])
%     subplot(1,5,2); dispmat(rc2,['peak, err = ',err2,' %'])
%     subplot(1,5,3); dispmat(rc3,['integral corrected, err = ',err3,' %'])
%     subplot(1,5,4); dispmat(rc4,['peak corrected, err = ',err4,' %'])
%     
%     subplot(1,5,5); 
%     plot(xs,...
%         [rc1(:,n/2),rc2(:,n/2),rc3(:,n/2),rc4(:,n/2),att(:,n/2)],...
%         'LineWidth',1.5);
%     legend('int','peak','int corr','peak corr','exact')
