function f = isotropic_source(n,m,thetas,type,sig,x,y)
% creates the boundary values for an isotropic source located at some point
% of the boundary of the unit square
% uses the function elem_isotropic_source

if x == 0
    [fene,fnne,fnnw,fwnw,fwsw,fssw,fsse,fese] = elem_isotropic_source(n,m,thetas,type,sig,y);
elseif y == 0
    [fnnw,fwnw,fwsw,fssw,fsse,fese,fene,fnne] = elem_isotropic_source(n,m,thetas,type,sig,x);
elseif x == 1
    [fwsw,fssw,fsse,fese,fene,fnne,fnnw,fwnw] = elem_isotropic_source(n,m,thetas,type,sig,1-y);
elseif y == 1
    [fsse,fese,fene,fnne,fnnw,fwnw,fwsw,fssw] = elem_isotropic_source(n,m,thetas,type,sig,1-x);
else
    fene = zeros(2*n,m);
    fnne = zeros(2*n,m);
    fnnw = zeros(2*n,m);
    fwnw = zeros(2*n,m);
    fwsw = zeros(2*n,m);
    fssw = zeros(2*n,m);
    fsse = zeros(2*n,m);
    fese = zeros(2*n,m);
end

f = [fene,fnne,fnnw,fwnw,fwsw,fssw,fsse,fese];

end


%--------------------------------------------------------------------------
function [fene,fnne,fnnw,fwnw,fwsw,fssw,fsse,fese] = elem_isotropic_source(n,m,thetas,type,sig,pos)
% creates the arrays for an isotropic source located on the left edge of
% the unit square

% any source involving a west-oriented direction will be zero
fnne = zeros(2*n,m);
fnnw = zeros(2*n,m);
fwnw = zeros(2*n,m);
fwsw = zeros(2*n,m);
fssw = zeros(2*n,m);
fsse = zeros(2*n,m);

% construction of fene
for j=1:m,
    fene(1:n,j) = predef_vec(type,n,sig,pos);
end
fene = [fene; zeros(size(fene))];

% construction of fese
fese = fftshift(flipud(fene),1);

fsse = zeros(2*n,m);
fnne = zeros(2*n,m);

switch type
    case 'gaussian'
        % construction of fsse
        for j=1:m,
            for k=1:floor(n*tan(thetas(j))),
                fsse(n+k,j) = exp(-((k-0.5)/n-pos*tan(thetas(j))).^2/sig^2);
            end
        end
        % construction of fnne
        for j=1:m,
            for k=1:floor(n*tan(thetas(j))),
                fnne(n+k,m+1-j) = exp(-((k-0.5)/n-(1-pos)*tan(thetas(j))).^2/sig^2);
            end
        end
    case 'dirac'
        % construction of fsse
        for j=1:m,
%             if sin(pi*(pos*tan(thetas(j))+1/2/n))==0
%                 fsse(n+floor(n*pos*tan(thetas(j))+0.5),j) = 1.;
%             else
%                 for k=1:floor(n*tan(thetas(j))),
%                     t = 1+pos*tan(thetas(j))-(1+1/2/n+(k-1)/n);
%                     %t = mod(t+0.5,1)-0.5;
%                     fsse(n+k,j) = sin(2*n*pi*t)/2/n/sin(pi*t);
%                 end
%             end
                
            fsse(n+floor(n*pos*tan(thetas(j))+0.5),j) = 1.;%+1 ?
        end
        % construction of fnne
        for j=1:m,
            fnne(n+floor(n*(1-pos)*tan(thetas(j))+0.5+1),m+1-j) = 1.;%+2 ?
        end
    otherwise
end

end