function [att, attRc, RTs,attErr] = itReconstrAtt(n,m,thetas,att,scat,nScat,...
    nIter,opts)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%--- INITIALIZATION

%--- parameters for the inverse Radon Transform
filter = opts.IRT.filter;
band = opts.IRT.band;
%--- number of radii for RT, make it even
nrRT = ceil(n*sqrt(2)) + mod(ceil(n*sqrt(2)),2);
%--- offset for the RT data
os = (nrRT-n)/2;

%--- diffusion coefficient
diff = makeDiff(n,opts.diff);

%--- angles for the inverse Radon Transform
angles = 180/pi*(thetas-pi/2);

%--- allocate
attRc = zeros(n,n,nIter+1);
RTs = zeros(nrRT,8*m,nIter+1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%--- MAIN

calibration = ~strcmp(opts.diff.type,'none') && ...
    (strcmp(opts.diff.rc,'peakCorr') || strcmp(opts.diff.rc,'intCorr'));

%--- if we use measurements without attenuation for the diffusion
%--- correction, we run ballistic measurements
if calibration
    disp('ballistic measurements without attenuation')
    corr = measForRcAtt(n,m,thetas,zeros(n),zeros(n),0,diff,0,opts.input);
    corr = measToRT(n,m,corr,opts);
end
    
%--- measurements for initial guess
disp('measurements for initial guess')
D = measForRcAtt(n,m,thetas,att,scat,nScat,diff,0,opts.input);

%--- initial guess
disp('initial guess')

if calibration
    RTs(:,:,1) = [zeros(os,8*m); measToRT(n,m,D,opts)-corr;zeros(os,8*m)];
else
    RTs(:,:,1) = [zeros(os,8*m); measToRT(n,m,D,opts);zeros(os,8*m)];
end

attRc(:,:,1) = (n/2)*myIRadon(RTs(:,:,1),filter,angles,band,n);

%--- iterative scheme

for it = 1:nIter,
    disp(['iteration ',num2str(it),' / ',num2str(nIter)])
    %--- measurements
    meas = measForRcAtt(n,m,thetas,attRc(:,:,it),scat,nScat,diff,1,...
        opts.input);
    
    if calibration
        RTs(:,:,it+1) = [zeros(os,8*m); ...
            measToRT(n,m,D-meas,opts)-corr; zeros(os,8*m)];
    else
        RTs(:,:,it+1) = [zeros(os,8*m); ...
            measToRT(n,m,D-meas,opts); zeros(os,8*m)];
    end

    attRc(:,:,it+1) = (n/2)*myIRadon(RTs(:,:,it+1),filter,angles,band,n);

end

attErr = zeros(1,nIter+1);
for it = 1:nIter+1,
    attRc(:,:,it) = attRc(:,:,it)-2*pi*scat;
    attErr(it) = norm(attRc(:,:,it)-att,'fro');
end

if norm(att,'fro')>0
    attErr = attErr/norm(att,'fro');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%--- DISPLAY

xs = -1+1/n:2/n:1-1/n;

figure;
subplot(2,3,1); dispmat(att,'true intrinsic attenuation (\sigma_a)')
subplot(2,3,2); dispmat(attRc(:,:,end),'last iteration')
subplot(2,3,3); dispmat(abs(att-attRc(:,:,end)),...
    'error at last iteration')
subplot(2,3,4); plot(xs,squeeze(attRc(:,n/2,:)));
title('iterations, "x=0"'); hold on; plot(xs,att(:,n/2),'k')
subplot(2,3,5); plot(xs,squeeze(attRc(n/2,:,:)));
title('iterations, "y=0"'); hold on; plot(xs,att(n/2,:),'k')
subplot(2,3,6); plot(attErr); title('L^2 relative error');
xlabel('iterations')


% xs = -1+1/n:2/n:1-1/n;
% for it = 1:nIter+1,
%     figure(70); subplot(1,nIter+1,it); dispmat(attRc(:,:,it));
%     figure(71); 
%     subplot(1,2,1); plot(xs,attRc(n/2,:,it)); hold on
%     subplot(1,2,2); plot(xs,attRc(:,n/2,it)); hold on
%     figure(72); 
%     subplot(2,nIter+1,it); contourf(xs,xs,abs(flipud(attRc(:,:,it)-att))); colorbar
%     subplot(2,nIter+1,nIter+1+it); dispmat(abs(attRc(:,:,it)-att));
%     figure(20); subplot(1,nIter+1,it); dispmat(RTs(:,:,it));
% end
% figure(71);
% subplot(1,2,1); title('attenuation - cut at y=0'); plot(xs,att(n/2,:));
% subplot(1,2,2); title('attenuation - cut at x=0'); plot(xs,att(:,n/2));
% % figure(72); title('reconstruction error')
% figure(20); title('Radon Data')
% if nIter
%     figure(73)
%     for it=1:nIter,
%         subplot(1,nIter,it); dispmat(attRc(:,:,it+1)-attRc(:,:,it));
%     end
%     title('differences of iterations')
% end

end
