function RT = measToRT(n,m,M,opts)

RT = zeros(n,8*m);

rs = -1+1/n:2/n:1-1/n;

%--- initial intensity of the pulses at the peak
switch opts.input.type
    case 'gaussian'
        I0 = 1/sqrt(2*pi)/opts.input.sig*(8*m/2/pi);
    case 'dirac'
        I0 = 8*m/2/pi;
    otherwise
        error('the value for the type variable is not valid')
end
%--- diffusion depths
diff = makeDiff(n,opts.diff);
sigout = sqrt(4/n*sum(diff,2));

% xs = -1+1/n:2/n:1-1/n;
% depths = 2*sqrt(1-xs.^2);

depths = 2/n*sum(opts.diff.domain,2);

switch opts.diff.type
    case 'FPB'
        sigout = sqrt(2*opts.diff.d*depths.^3/3);
    case 'linear'
        sigout = sqrt(opts.diff.d*depths.^2);
    otherwise
end

for th = 1:8*m,
    for indR = 1:n,
        
        temp = M(indR,th,:);
        
        switch opts.diff.type
            case {'linear','FPB'}
                switch opts.diff.rc
                    case 'int'
                        if sigout(indR)
                            RT(indR,th) = -log(2/n*sum(temp)/I0);
                        else
                            RT(indR,th) = -log(temp(indR)/I0*2/n);
                        end
                    case 'peak'
                        if sigout(indR)
                            RT(indR,th) = -log(sigout(indR)*...
                                sqrt(2*pi)*temp(indR)/I0);
                        else
                            RT(indR,th) = -log(temp(indR)/I0);
                        end
                    case 'intCorr'
                        RT(indR,th) = -log(sum(temp));
                    case 'peakCorr'
                        RT(indR,th) = -log(temp(indR));
                    case 'nodiff'
                        RT(indR,th) = -log(temp(indR)/(I0*n/2));
                    otherwise
                        error('the reconstruction variable is not valid')
                end
            case 'none'
                RT(indR,th) = -log(temp(indR)/(I0*n/2));
            otherwise
                error('the value for opts.diff.type is not valid.')
        end
        
%         if abs(rs(indR))>=0.90
%             RT(indR,th) = 0;
%         end
    end
end

end
