function im2 = rotateImage(im,theta)
% for an image of even size n, this function creates a rotated version of
% that image. 

theta = mod(theta,2*pi);

if theta<=pi/4,
    im2 = rotateImageElem(im,theta);
elseif theta<=pi/2,
    im2 = flipud(rotateImageElem(flipud(rot90(im,-1)),pi/2-theta));
elseif theta<=3*pi/4,
    im2 = rotateImageElem(rot90(im,-1),theta-pi/2);
elseif theta<=pi,
    im2 = flipud(rotateImageElem(flipud(rot90(im,-2)),pi-theta));
elseif theta<=5*pi/4
    im2 = rotateImageElem(rot90(im,-2),theta-pi);
elseif theta<=3*pi/2
    im2 = flipud(rotateImageElem(flipud(rot90(im,-3)),3*pi/2-theta));
elseif theta<=7*pi/4
    im2 = rotateImageElem(rot90(im,-3),theta-3*pi/2);
else
    im2 = flipud(rotateImageElem(flipud(im),2*pi-theta));
end

end  

function im2 = rotateImageElem(im,theta)

if theta==0
    im2 = im;
elseif theta<=pi/4
    im2 = slantResample(im,tan(theta),1/cos(theta));
    im2 = rot90(slantResample(rot90(im2),sin(theta),cos(theta)),-1);
else
    error(['theta = ',num2str(theta),' is not a valid angle.']);
end

end

function res = slantResample(im,t,h)

res = zeros(size(im));

if norm(im,'fro')==0
    return
else
    [nr,nc] = size(im);
  
    shifts = (-1+1/nc:2/nc:1-1/nc)*t/2;
    N = 2*nr;
    %--- slant params
    temp1 = exp(-i*(pi/N)*(-N+1)*(0:N-1)');
    % temp2 = conj(temp1);
    
    %--- resampling params (precomputations for the FRFT of coeff -h/N
    z = exp(-i*pi*(0:N).^2*h/N); z = [z,z(N:-1:2)]; Fz = fft(z).';
    factors = exp(i*pi*h/N*(0:N-1).^2).'; 
    ns = nr-1/2 - h*(nr/2-1/2);
    
    % preproc = exp(-i*(pi/N)*(-N+1)*(0:N-1).');
    % temp2.*preproc = 1
    midproc = factors.*exp(i*(pi/N)*((-N+1):2:(N-1)).'*ns);
    postproc = factors(1:nr).*exp(i*(pi/N)*(-N+1)*h*(0:(N/2-1)).');
  
    phases = -i*pi/2*((-N+1):2:(N-1))';
    
    for c=1:nc,
        % slant
        colTempN = [zeros(nr/2,1); im(:,c); zeros(nr/2,1)].*temp1;
        colTempN = fft(colTempN).*exp(shifts(c)*phases);
%         colTemp = ifft(colTempN).*temp2;
        
        % resampling
               
%         colTemp = ifft(fft(fft(colTempN).*midproc,2*N).*Fz);
        colTemp2N = ifft(fft(colTempN.*midproc,2*N).*Fz);
        res(:,c) = 1/N*postproc.*colTemp2N(1:nr);
    end
    
    if all(all(isreal(im)))
        res = real(res);
    end
    
    %% TODO : try out the decimation scheme in C
end   
    
end