clc; clear; close all

n = 32;
m = n/8; % number of direction between 0 and pi/4
thetas = pi/4*(1/2/m:1/m:8-1/2/m);

xs = -1+1/n:2/n:1-1/n;

%--- characteristic function of the domain
domain = myMap('disk',n,1,0,0);

%--- intrisic attenuation
% att = zeros(n);
att = myMap('mickey',n,0.02,12,0.2);%+0.3*domain;
% att = myMap('nicebumps',n,0.1,5,0.5);%+0.3*domain;
% att = domain;

%--- scattering
coeffScat = 0.2;
% scat = coeffScat*myMap('gaussian',n,1.5/n,0.5,0.5);
% scat = coeffScat*myMap('mickey',n,0.01,16,0.2);
scat = coeffScat*myMap('nicebumps',n,0.1,3,0.4);
% scat = coeffScat*myMap('disk',n,0.03,0.5,0)...
%     + coeffScat*myMap('disk',n,0.03,-0.5,0);
% scat = coeffScat*(myMap('disk',n,0.5,0,0)...
%     -myMap('disk',n,0.4,0,0));
% scat = coeffScat*myMap('disk',n,0.02,0,0);
% scat = coeffScat*myMap('phantom',n,0.06,3,0.5);
% scat = coeffScat*(myMap('gaussian',n,0.3,0,0) + ...
%     myMap('gaussian',n,0.125,0.5,0.5) + ...
%     myMap('gaussian',n,0.125,-0.5,-0.5));
% scat = zeros(n);

% cropping the coefficients to the domain
scat = scat.*domain;

nIter = 0; % number of iterations for the reconstruction scheme
nScat = 3; % number of scattering terms for the forward solver

%--- indices thIn, thOut
% indsIn = [1,1,1,1];
% indsOut = indsIn + [m/4,ceil(2*m/3),ceil(4*m/3),2*m];
indsIn = 1;
indsOut = indsIn + m;

%--- input options
opts.input.type = 'dirac';
opts.input.sig = 4.5/n;

%--- diffusion types
%     opts.diff.type = 'linear'; opts.diff.d = tan(pi/4/m)^2;
    opts.diff.type = 'FPB'; opts.diff.d = 0.00;
  %   opts.diff.type = 'none'; opts.diff.sig = 0;

%--- domain for diffusion
opts.diff.domain = domain;
%--- recontruction formula for attenation
opts.diff.rc = 'intCorr'; % 'int', 'peak', 'intCorr', 'peakCorr'
    
% corrector for scattering reconstruction based 
% on unitary scattering coefficient
opts.corr = 0;

%--- options for the inverse Radon Transform    
opts.IRT.filter = 'ram-lak'; % filter for the inverse Radon Transform
opts.IRT.band = 0.9; % frequency truncation for the IRT


% % =========== iterated scattering reconstruction
% 
    %--- compute initial guess : yes
    opts.computeIG = 1;
    %--- type of iterative scheme ('initial' or 'modified')
    opts.typeIter = 'modified';
        
    %--- reconstructions without calibration
    [scatRc, scatErr, attRc, attErr] = itReconstrScat(n,m,thetas,...
        att,scat,nScat,nIter,indsIn,indsOut,opts);

    %--- reconstructions with calibration
    opts.corr = 1;

    [scatRc2, relErr2, attRc2, attErr2] = itReconstrScat(n,m,thetas,...
        att,scat,nScat,nIter,indsIn,indsOut,opts);
    
    %--- display
    close all
    figure(1)
    subplot(1,4,1); dispmat(scatRc(:,:,end), 'A: without corrector')
    subplot(1,4,2); dispmat(scatRc2(:,:,end), 'B: with corrector')
    subplot(1,4,3); plot(xs,[scatRc(:,n/2,end),scatRc2(:,n/2,end),scat(:,n/2)],...
        'LineWidth',1.5); title('C: cut at x=0')
    legend('no corrector','corrector','exact')
    subplot(1,4,4);
    plot(xs,[scatRc(n/2,:,end).',scatRc2(n/2,:,end).',scat(n/2,:).'],...
        'LineWidth',1.5); title('D: cut at y=0')
    legend('no corrector','corrector','exact')

