function res = solveODE(n,input,attRot,srcRot,diffRot)

res = zeros(n);

scheme = 0; % 0:EE, 1:RK2

boolomega = any(any(~isreal(attRot)));

if boolomega
    phase = exp(2*i/n*imag(attRot));
    attRot = real(attRot);
else
    phase = 0;
end

switch scheme
    case 0 % EE

        if any(any(diffRot))
            %--- first column
            if any(input)
                res(:,1) = (1-1/n*attRot(:,1)).*input + n*diffRot(:,1) ...
                    .*([0;input(1:n-1)] + [input(2:end);0] - 2*input);
            end

            %--- other columns
            for j=2:n,
                res(:,j) = (1-2/n*attRot(:,j-1)).*res(:,j-1) ...
                    + 2/n*srcRot(:,j-1) + n/2*diffRot(:,j-1) ...
                    .*([0;res(1:n-1,j-1)] + [res(2:end,j-1);0] - ...
                    2*res(:,j-1));
                if boolomega
                    res(:,j) = res(:,j).*phase(:,j-1);
                end
            end
        else
            %--- first column
            if any(input)
                res(:,1) = (1-1/n*attRot(:,1)).*input;
            end

            %--- other columns
            for j=2:n,
                res(:,j) = (1-2/n*attRot(:,j-1)).*res(:,j-1) + ...
                    2/n*srcRot(:,j-1);
                if boolomega
                    res(:,j) = res(:,j).*phase(:,j-1);
                end
            end
        end

    case 1 % RK2
        
        if any(any(diffRot))
            %--- first column
            if any(input)
                res(:,1) = (1-1/n*attRot(:,1)).*input + n*diffRot(:,1) ...
                    .*lapl(input);
            end

            %--- other columns
            for j=2:n,
                
                k1 = -attRot(:,j-1).*res(:,j-1) + n^2/4*...
                    diffRot(:,j-1).*lapl(res(:,j-1)) + srcRot(:,j-1);
                k2 = -attRot(:,j).*(res(:,j-1)+ 2/n*k1) + n^2/4*...
                    diffRot(:,j).*lapl(res(:,j-1)+ 2/n*k1) + srcRot(:,j);
                
                res(:,j) = res(:,j-1) + 1/n*(k1 + k2);
            end
        else
            %--- first column
            if any(input)
                res(:,1) = (1-1/n*attRot(:,1)).*input;
            end

            %--- other columns
            for j=2:n,
                k1 = -attRot(:,j-1).*res(:,j-1) + srcRot(:,j-1);
                k2 = -attRot(:,j).*(res(:,j-1)+2/n*k1) + srcRot(:,j);
                
                res(:,j) = res(:,j-1) + 1/n*(k1 + k2);
            end
        end
end

if max(max(abs(res)))>n^3
    error('blow up !')
end

end

% function lapl
%--------------------------------------------------------------------------
function vecOut = lapl(vecIn)
    if any(vecIn)
        vecOut = [0;vecIn(1:end-1)] + [vecIn(2:end);0] - 2*vecIn;
    else
        vecOut = vecIn;
    end
end

